from pathlib import Path
import re
from typing import Optional
from dataclasses import dataclass
import toml
from rich.console import Console


@dataclass
class Script:
    name: str
    comments: list[str]
    inline_comment: Optional[str]
    command: Optional[str]


header_pattern = re.compile(r"^\s*\[(.*?)\]\s*$")
key_pattern = re.compile(r"""^([^\s=]+)\s*=.*(?<!['\"]).*""")
comment_pattern = re.compile(r"""(#.*)$""")

TOOL_HEADER = "tool"
PPP_HEADER = "ppp"
SCRIPTS_HEADER = "scripts"
FULL_HEADER = f"{TOOL_HEADER}.{PPP_HEADER}.{SCRIPTS_HEADER}"


def extract_toml_headers(line: str) -> Optional[str]:
    match = header_pattern.match(line)
    if not match:
        return None
    return match.group(1).strip()


def extract_comment(line: str) -> Optional[str]:
    if line.startswith("#"):
        return line[1:].strip()
    return None


def unwrap_quotes(key: str) -> str:
    """Unwraps quotes from the key if they are present."""
    if key.startswith("'") and key.endswith("'"):
        return key[1:-1]
    elif key.startswith('"') and key.endswith('"'):
        return key[1:-1]
    return key


def extract_entry(line: str) -> Optional[tuple[str, Optional[str]]]:
    """If entry is detected, return its key and optional inline comment as a tuple."""
    match = key_pattern.match(line)
    if not match:
        return None
    key = match.group(1).strip()
    key = unwrap_quotes(key)
    if key.startswith(("'", '"')) and key.endswith(("'", '"')):
        key = key[1:-1]

    comment_match = comment_pattern.search(line)

    if not comment_match:
        return (key, None)

    inline_comment = comment_match.group(0)
    if inline_comment is not None:
        inline_comment = inline_comment[1:].strip()
    return (key, inline_comment)


def extract_scripts(file_path: Path, console: Console) -> list[Script]:
    """
    Extracts scripts from tool.pp.scripts section within pyproject TOML file, returning a
    list of Script objects.
    """
    found_header: bool = False
    comments: list[str] = []
    scripts: list[Script] = []

    with open(file_path, "r") as file:
        for line in file:
            line = line.strip()
            if not found_header:
                header = extract_toml_headers(line)
                if header is not None and header.startswith(FULL_HEADER):
                    found_header = True
                continue
            else:
                if line.strip() == "" or extract_toml_headers(line) is not None:
                    # Reached the end of the scripts section or a new header
                    break
            comment = extract_comment(line)
            if comment:
                # It's a comment, aggregate it for the next script
                comments.append(comment)
                continue
            entry = extract_entry(line)
            if entry is not None:
                key, inline_comment = entry
                script = Script(
                    name=key,
                    comments=comments,
                    inline_comment=inline_comment,
                    command=None,
                )
                comments = []
                scripts.append(script)

    # I won't write a TOML parser, and we can't trust regex to parse TOML correctly. So we
    # use toml library and match the scripts by keys
    with open(file_path, "r") as file:
        toml_data = toml.load(file)
        toml_scripts = (
            toml_data.get(TOOL_HEADER, {}).get(PPP_HEADER, {}).get(SCRIPTS_HEADER, None)
        )
        if toml_scripts is None:
            console.print(
                "Specify your scripts in [bold][tool.ppp.scripts][/bold] section in [bold]pyproject.toml[/bold]"
            )
            return []
        for key, value in toml_scripts.items():
            for script in scripts:
                if script.name == key:
                    script.command = value
                    break
    return scripts



from pathlib import Path
from unittest.mock import Mock
from rich.console import Console

from ppp.toml_service import Script, extract_scripts
from ppp.tests.lib import get_test_directory


def get_scripts() -> list[Script]:
    return [
        Script(
            name="docker-up",
            command="docker run -it -v bob:the_builder nvm",
            comments=["Run a docker image with http server"],
            inline_comment="[start|stop]",
        ),
        Script(
            name="type",
            command="echo",
            comments=[],
            inline_comment="text",
        ),
        Script(
            name="shell",
            command="echo $SHELL",
            comments=[],
            inline_comment="text",
        ),
    ]


def test() -> None:
    config = get_test_directory() / "test_pyproject.toml"
    mock_console = Mock(spec=Console)  # Create a mock Console
    scripts: list[Script] = extract_scripts(
        Path(config),
        console=mock_console,
    )
    assert scripts == get_scripts()
    print("scripts", scripts)



import sys
import subprocess
from pathlib import Path
from rich.console import Console
from typing import Any, Optional
from ppp.toml_service import (
    Script,
    extract_scripts,
)


console = Console()


def list_scripts(scripts: list[Script]) -> None:
    console.print("[bold cyan]Available scripts:[/bold cyan]")
    for script in scripts:
        console.print(f"[bold yellow]{script.name}[/bold yellow]", end=" ")
        if script.inline_comment is not None and script.inline_comment != "":
            console.print(script.inline_comment, end="", style="dim", markup=False)
        if len(script.comments) > 0:
            console.print(" - " + "\n".join(script.comments))
        else:
            console.print("")


def run_script(scripts: list[Script], script_name: str, args: Any) -> None:
    print("Running script:", script_name)

    found_script: Optional[Script] = None

    for script in scripts:
        if script.name == script_name:
            found_script = script
            break

    if found_script is None:
        console.print(f"[bold red]Error: Script '{script_name}' not found![/bold red]")
        sys.exit(1)

    if not found_script.command:
        console.print(
            f"[bold red]Error: Script '{script_name}' doesn't have a command![/bold red]"
        )
        sys.exit(1)

    subprocess.run(
        " ".join([found_script.command, *args]), shell=True, executable="/bin/zsh"
    )


def show_help() -> None:
    console.print("[bold green]Usage:[/bold green]")
    console.print(
        "  [bold yellow]ppp run <script> [args][/bold yellow] - Run a script with optional arguments"
    )
    console.print(
        "  [bold yellow]ppp list[/bold yellow] - List available scripts with descriptions"
    )
    console.print("  [bold yellow]ppp help[/bold yellow] - Show this help message")


def main() -> None:
    if len(sys.argv) < 2:
        show_help()
        sys.exit(1)

    command = sys.argv[1]

    pyproject_path = Path("pyproject.toml")
    if not pyproject_path.exists():
        console.print("[bold red]Error: pyproject.toml not found![/bold red]")
        sys.exit(1)

    scripts = extract_scripts(pyproject_path, console=console)

    if command == "list":
        list_scripts(scripts)
    elif command == "help":
        show_help()
    elif command == "run" and len(sys.argv) > 2:
        script_name = sys.argv[2]
        args = sys.argv[3:]
        run_script(scripts, script_name, args)
    else:
        console.print(
            "[bold red]Invalid command. Use 'ppp help' for usage information.[/bold red]"
        )
        sys.exit(1)


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="ppp-run",
    version="0.1",
    packages=find_packages(),
    install_requires=["toml", "rich"],
    entry_points={
        "console_scripts": [
            "ppp=ppp.cli:main",
        ],
    },
    author="Konrad Frysiak",
    description="A CLI tool to run scripts/aliases from pyproject.toml",
)
